@@ -2,7 +2,7 @@ class ControllerGenerator < Rails::Generator::NamedBase
   def manifest
     record do |m|
       # Check for class naming collisions.
-      m.class_collisions class_path, "#{class_name}Controller", "#{class_name}ControllerTest", "#{class_name}Helper"
+      m.class_collisions "#{class_name}Controller", "#{class_name}ControllerTest", "#{class_name}Helper"
 
       # Controller, helper, views, and test directories.
       m.directory File.join('app/controllers', class_path)
@@ -28,9 +28,8 @@ class ControllerGenerator < Rails::Generator::NamedBase
 
       # View template for each action.
       actions.each do |action|
-        path = File.join('app/views', class_path, file_name, "#{action}.rhtml")
-        m.template 'view.rhtml',
-          path,
+        path = File.join('app/views', class_path, file_name, "#{action}.html.erb")
+        m.template 'view.html.erb', path,
           :assigns => { :action => action, :path => path }
       end
     end
